/**=================================================================================================
Hochschule Luzern - Technik & Architektur                                   Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                     http://hslu.ximit.ch
----------------------------------------------------------------------------------------------------
Project     : Uebung_5-1
Name        : matrix.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-27v1.0
Description : Matrix erstellen
==================================================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include "matrix.h"

int** createMatrix(int rows, int cols, int value) {
  int row;                // used for iteration
  int** matrix = NULL;    // placeholder for matrix
  
  // allocate memory
  matrix = (int**) malloc(rows*sizeof(int*));
  if(matrix != NULL) {    // if succesfull allocated
    
    // allocate rows
    for(row=0; row<rows; row++) {
      matrix[row] = (int*) malloc(cols*sizeof(int));
      if(matrix[row] == NULL) {
        printf("Unable to allocate memory!\n");
        // free if unsuccesfull allocation
        freeMatrix(matrix, row);
        return NULL;
      }
    }
  
    // fill matrix with values
    fillMatrix(matrix, rows, cols, value);
  }
  else printf("Unable to allocate memory!\n");
  
  return matrix;
}

void fillMatrix(int** matrix, int rows, int cols, int value) {
  int row, col;   // used for iteration
  
  // fill matrix
  for(row = 0; row < rows; row++)
    for(col = 0; col < cols; col++)
      matrix[row][col] = value;
}

void printMatrix(int* const* matrix, int rows, int cols) {
  int row, col;   // used for iteration
  
  for(row = 0; row < rows; row++) {
    printf("{");   // print row boundary
    for(col = 0; col < cols; col++) {
      // print cell. separator only if first cell of row
      printf(col == 0 ? "%6d" : ",%6d", matrix[row][col]);
    }
    printf("}\n");   // print row boundary
  }
}

void freeMatrix(int** matrix, int rows) {
  int row;        // used for iteration
  
  // free columns first
  for (row = 0; row < rows; row++){
    free(matrix[row]);
  }
  
  // free rows
  free(matrix);
}
