/**=================================================================================================
Hochschule Luzern - Technik & Architektur                                   Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                     http://hslu.ximit.ch
----------------------------------------------------------------------------------------------------
Project     : Uebung_5-3
Name        : semaphore.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-27v1.0
Description : Zeiger auf Funktion / Threads
==================================================================================================*/
#include <stdio.h> 
#include <string.h>
#include <pthread.h> 
#include "semaphore.h" 

static int threadRunning = 0;
static SemaphorStates_t state = RED;
static pthread_t threadId; // Thread stuff 
static pthread_attr_t threadAttr;
int (*f)(SemaphorStates_t phase);
int sleepTime;

/** 
 * Printout current thread state 
 */
static void printThreadState(int sleepTime) {
    char stateStr[64];

    switch (state) {
        case GREEN: strcpy(stateStr, "GREEN");
            break;
        case GREEN_BLINKING: strcpy(stateStr, "GREEN BLINKING");
            break;
        case YELLOW: strcpy(stateStr, "YELLOW");
            break;
        case RED: strcpy(stateStr, "RED");
            break;
        default: strcpy(stateStr, "UNDEFINED");
            break;
    }
    printf("The semaphor is %-14s for %2d seconds\n", stateStr, sleepTime);
    fflush(stdout);
}

static void* threadFunction(void* args) {
    while (threadRunning) {
        switch (state) {
            case GREEN:
                state = GREEN_BLINKING;
                break;
            case GREEN_BLINKING:
                state = YELLOW;
                break;
            case YELLOW:
                state = RED;
                break;
            case RED:
                state = GREEN;
                break;
            default: // Should not happen, but added error recovery 
                state = RED;
                break;
        }

        if (f != NULL) {
            sleepTime = f(state);
        }

        printThreadState(sleepTime);
        sleep(sleepTime);
    }
    printf("Semaphor thread stopped\n");

    return NULL;
}

void setSemaphorCallbackFunction(int (*b) (SemaphorStates_t phase)) {
    f = b;
}

void startSemaphore() {
    threadRunning = 1;

    // Thread starten 
    pthread_attr_init(&threadAttr);
    // Hier wird als drittes Argument ein Zeiger auf die Thread-Funktion übergeben. 
    pthread_create(&threadId, &threadAttr, threadFunction, NULL);
}

void stopSemaphor() {
    threadRunning = 0;

    // Wait until thread is finished 
    pthread_join(threadId, NULL);
    pthread_attr_destroy(&threadAttr);
}
