/**=================================================================================================
Hochschule Luzern - Technik & Architektur                                   Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                     http://hslu.ximit.ch
----------------------------------------------------------------------------------------------------
Project     : Uebung_5-3
Name        : semaphore.h
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-27v1.0
Description : Zeiger auf Funktion / Threads
==================================================================================================*/
#ifndef SEMAPHORE_H
#define	SEMAPHORE_H

/** 
 * Implementation of a semaphor with a thread. 
 * Uses a callback function to define the various times. 
 */ 
typedef enum SemaphorStates_ { 
    GREEN = 1, 
    GREEN_BLINKING, 
    YELLOW, 
    RED 
} SemaphorStates_t; 
 
/** 
 * Set the callback function that defines the time of each phase. 
 * @param b pointer to function that returns the phase length in seconds. 
 *        The function gets as Parameter the current semaphore state. 
 */ 
void setSemaphorCallbackFunction(int (*b) (SemaphorStates_t phase)); 
 
/** 
 * Starts the semaphor. 
 * Within a thread, the semaphor changes its state from GREEN to GREEN_BLINKING 
 * to YELLOW to RED to GREEN to ... . 
 * After each state change, the callbackfunction defined by setCallbackFunction(...)    
 * will be called to get the length of the corresponding phase. 
 */ 
void startSemaphore(); 
 
/** 
 * Stop the semaphor. 
 */ 
void stopSemaphor(); 

/**
 * Definiert die Semaphorezeiten
 */
int getPhaseTime(SemaphorStates_t phase);

#endif	/* SEMAPHORE_H */
